"""Functions to create timestamp according to SSB standard."""


def _check_even(elements: list[int]) -> bool:
    """Function to check if number is even."""
    return len(elements) % 2 == 0


def _check_valid_day(day: int) -> None:
    """Function to check that day arg is valid."""
    if day > 31:
        raise ValueError(
            f"The arg for day is bigger than possible max is 31 you have: {day}."
        )
    if day < 1:
        raise ValueError(
            f"The arg for day is smaller than possible min is 1 you have: {day}."
        )


def _check_valid_week(week: int) -> None:
    """Function to check that day arg is valid."""
    if week > 52:
        raise ValueError(
            f"The arg for week is bigger than possible max is 52 you have: {week}."
        )
    if week < 1:
        raise ValueError(
            f"The arg for week is smaller than possible min is 1 you have: {week}."
        )


def _check_valid_month(month: int) -> None:
    """Function to check that month arg is valid."""
    if month > 12:
        raise ValueError(
            f"The arg for month is bigger than possible max is 12 you have: {month}."
        )
    if month < 1:
        raise ValueError(
            f"The arg for month is smaller than possible min is 1 you have: {month}."
        )


def _check_valid_term(term: int) -> None:
    """Function to check that day arg is valid."""
    if term > 6:
        raise ValueError(
            f"The arg for term is bigger than possible max is 6 you have: {term}."
        )
    if term < 1:
        raise ValueError(
            f"The arg for term is smaller than possible min is 1 you have: {term}."
        )


def _check_valid_quarter(quarter: int) -> None:
    """Function to check that day arg is valid."""
    if quarter > 4:
        raise ValueError(
            f"The arg for quarter is bigger than possible max is 4 you have: {quarter}."
        )
    if quarter < 1:
        raise ValueError(
            f"The arg for quarter is smaller than possible min is 1 you have: {quarter}."
        )


def _check_valid_trimester(trimester: int) -> None:
    """Function to check that day arg is valid."""
    if trimester > 3:
        raise ValueError(
            f"The arg for trimester is bigger than possible max is 3 you have: {trimester}."
        )
    if trimester < 1:
        raise ValueError(
            f"The arg for trimester is smaller than possible min is 1 you have: {trimester}."
        )


def _check_valid_half_year(half_year: int) -> None:
    """Function to check that day arg is valid."""
    if half_year > 2:
        raise ValueError(
            f"The arg for half_year is bigger than possible max is 2 you have: {half_year}."
        )
    if half_year < 1:
        raise ValueError(
            f"The arg for half_year is smaller than possible min is 1 you have: {half_year}."
        )


def _check_valid_year(year1: int, year2: int | None = None) -> None:
    """Function to check that year is valid."""
    if len(str(year1)) != 4:
        raise ValueError(
            f"Any valid year should be length 4, you have: {len(str(year1))}, maybe you should check if year: {year1} is correct."
        )
    if year2:
        if len(str(year2)) != 4:
            raise ValueError(
                f"Any valid year should be length 4, you have: {len(str(year2))}, maybe you should check if year: {year2} is correct."
            )

        if year1 > year2:
            raise ValueError(
                f"The order of args is start date and then end date. Therefore first year arg can not be bigger than the last. Your args are start year:{year1}  end year:{year2}."
            )


def _check_valid_args(*args: int, frequency: str) -> None:
    """Function to check if valid args."""
    if len(args) == 2:
        _check_valid_year(args[0])
        if frequency == "W":
            _check_valid_week(args[1])
        if frequency == "M":
            _check_valid_month(args[1])
        if frequency == "B":
            _check_valid_term(args[1])
        if frequency == "Q":
            _check_valid_quarter(args[1])
        if frequency == "T":
            _check_valid_trimester(args[1])
        if frequency == "H":
            _check_valid_half_year(args[1])
    elif len(args) == 4:
        _check_valid_year(args[0], args[2])
        if frequency == "W":
            _check_valid_week(args[1])
            _check_valid_week(args[3])
        if frequency == "M":
            _check_valid_month(args[1])
            _check_valid_month(args[3])
        if frequency == "B":
            _check_valid_term(args[1])
            _check_valid_term(args[3])
        if frequency == "Q":
            _check_valid_quarter(args[1])
            _check_valid_quarter(args[3])
        if frequency == "T":
            _check_valid_trimester(args[1])
            _check_valid_trimester(args[3])
        if frequency == "H":
            _check_valid_half_year(args[1])
            _check_valid_half_year(args[3])
    else:
        raise ValueError(
            f"You have the wrong number of args, youre args are {args}. You can have 2 or 4 for frequency: {frequency}"
        )


def _check_frequency_suport(frequency: str) -> None:
    """Function to check if frequency requested is supported."""
    if frequency not in ["Y", "Q", "B", "M", "D", "W", "T", "H"]:
        raise ValueError(
            f"The function does not support frequency: {frequency} yet. Please use one the supported ones: Y,Q,B,M,D,W,T,H"
        )


def _get_timestamp_daily(*args: int) -> str | None:
    """Function to create timestamp if frequency is daily."""
    if len(args) == 3:
        _check_valid_year(args[0])
        _check_valid_month(args[1])
        _check_valid_day(args[2])
        return f"p{args[0]}-{args[1]:02}-{args[2]:02}"
    elif len(args) == 6:
        _check_valid_year(args[0], args[3])
        _check_valid_month(args[1])
        _check_valid_day(args[2])
        _check_valid_month(args[4])
        _check_valid_day(args[5])
        return (
            f"p{args[0]}-{args[1]:02}-{args[2]:02}_p{args[3]}-{args[4]:02}-{args[5]:02}"
        )
    else:
        raise ValueError(f"Ikke gyldig mende args, du har antall:{len(args)}")


def _get_timestamp_yearly(*args: int) -> str | None:
    """Function to create timstamp if frequency is yearly."""
    if len(args) == 2:
        _check_valid_year(args[0], args[1])
        return f"p{args[0]}_p{args[1]}"
    else:
        raise ValueError(
            f"You have the wrong amount of args, you can have two you have args: {args}"
        )


def _get_timestamp_special(*args: int, frequency: str) -> str | None:
    """Function to create timestamp if frequency is not Y or D."""
    _check_valid_args(*args, frequency=frequency)

    if frequency in ["M", "W"]:
        if frequency == "M":
            frequency = "-"
        if len(args) == 2:
            return f"p{args[0]}{frequency}{args[1]:02}"
        elif len(args) == 4:
            return (
                f"p{args[0]}{frequency}{args[1]:02}_p{args[2]}{frequency}{args[3]:02}"
            )
        else:
            return None
    else:
        if len(args) == 2:
            return f"p{args[0]}{frequency}{args[1]}"
        elif len(args) == 4:
            return f"p{args[0]}{frequency}{args[1]}_p{args[2]}{frequency}{args[3]}"
        else:
            return None


def get_ssb_timestamp(*args: int, frequency: str = "M") -> str | None:
    r"""Function to create a string in ssb timestamp format.

    Args:
        args: Up to six arguments with int, to create timestamp for.
        frequency: Letter for which frequency the data is, Y for year etc.

    Returns:
        string|None: Returns time stamp in ssb format.

    Raises:
        ValueError: Raises error for wrong values in args.

    Example:
        >>> get_ssb_timestamp(2024,8,1, frequency='D')
        'p2024-08-01'
    """
    _check_frequency_suport(frequency)

    if len(args) > 6:
        raise ValueError(
            f"Du kan ikke ha flere enn seks argumenter for å lage en ssb timestamp. Du har: {len(args)}"
        )
    elif all(arg is None for arg in args):
        return None
    elif not args[0]:
        raise ValueError(
            "Mangler start år, timestamp blir da None. Vurder å fylle inn start år."
        )
    elif len(args) == 1 and frequency == "Y":
        _check_valid_year(args[0])
        return f"p{args[0]}"
    else:

        valid_args = [arg for arg in args if arg]

        if frequency == "D":
            return _get_timestamp_daily(*valid_args)

        if not _check_even(valid_args) or len(valid_args) > 4:
            print(
                f"For frekvens '{frequency}', må du ha enten to eller fire argumenter. Du har:",
                len(valid_args),
            )
            return None
        else:
            if frequency == "Y":
                return _get_timestamp_yearly(*valid_args)
            else:
                return _get_timestamp_special(*valid_args, frequency=frequency)



"""Function for date prompts in python.

The functions should be used to prompt which period to read files from og which period to run a statistic for.
"""

import re
from calendar import monthrange
from datetime import datetime
from typing import Any

import pendulum


def _input_valid_int() -> int:
    """Input function for valid int.

    Returns:
        int: Return only valid int.
    """
    # Get the desired year from the user
    while True:
        user_input = input("Input: ")

        try:
            valid_int = int(user_input)
            break  # Break the loop if a valid integer is entered
        except ValueError:
            print("Vennligst skriv inn et gyldig tall som f.eks.", 42)

    return valid_int


def input_year() -> int:
    """Input function for year.

    Returns:
        int: Year as int
    """
    # Get the desired year from the user
    now = pendulum.now("Europe/Oslo").year
    print("Skriv inn år i format YYYY som", now)
    while True:
        year = _input_valid_int()
        if year == now:
            return year
        else:
            print("Er du sikker på at du skal kjøre statistikk for dette året?", year)
            if input("y/n") == "y":
                return year


def input_month() -> int:
    """Input function for month.

    Returns:
        int: month
    """
    # Get the desired month from the user
    print("Skriv inn måned i format m, som:", 8)
    while True:
        month = _input_valid_int()
        if 1 <= month <= 12:
            return month
        else:
            print("Ikke en gyldig måned, vennligst skriv inn et tall fra 1 til 12.")


def input_term() -> int:
    """Input function for term.

    Returns:
        int: term
    """
    print("Skriv inn termin i format b, som:", 3)
    while True:
        term = _input_valid_int()
        if 1 <= term <= 6:
            return term
        else:
            print("Ikke en gyldig termin, vennligst skriv inn et tall fra 1 til 6.")


def input_quarter() -> int:
    """Input function for quarter.

    Returns:
        int: quarter
    """
    print("Skriv inn kvartal i format q, som:", 2)
    while True:
        quarter = _input_valid_int()
        if 1 <= quarter <= 4:
            return quarter
        else:
            print("Ikke et gyldig kvartal, vennligst skriv inn et tall fra 1 til 4.")


def input_trimester() -> int:
    """Input function for trimester.

    Returns:
        int: trimester
    """
    print("Skriv inn trimester i format t, som:", 2)
    while True:
        trimester = _input_valid_int()
        if 1 <= trimester <= 3:
            return trimester
        else:
            print("Ikke et gyldig trimester, vennligst skriv inn et tall fra 1 til 3.")


def input_week() -> int:
    """Input function for week.

    Returns:
        int: week
    """
    print("Skriv inn week i format w, som:", 52)
    while True:
        week = _input_valid_int()
        if 1 <= week <= 52:
            return week
        else:
            print("Ikke en gyldig uke, vennligst skriv inn et tall fra 1 til 52.")


def days_in_month(year: int, month: int) -> list[str]:
    """Function to get number of days in month.

    Args:
        year: Year.
        month: Month.

    Returns:
        list: List with days in month.
    """
    num_days = monthrange(year, month)[
        1
    ]  # Get the number of days in the specified month
    days_list = [f"{year}-{month:02d}-{day:02d}" for day in range(1, num_days + 1)]
    return days_list


def extract_start_end_dates(file_name: str) -> tuple[datetime, datetime]:
    """Function to extract start and end dates from file name.

    Args:
        file_name: String value with name of file.

    Returns:
        tuple: Tuple with two datetime objects
    """
    match = re.search(r"_p(\d{4})-(\d{2})-(\d{2})_p(\d{4})-(\d{2})-(\d{2})_", file_name)
    if match:
        start_date = datetime.strptime(
            f"{match.group(1)}-{match.group(2)}-{match.group(3)}", "%Y-%m-%d"
        )
        end_date = datetime.strptime(
            f"{match.group(4)}-{match.group(5)}-{match.group(6)}", "%Y-%m-%d"
        )
    return start_date, end_date


def months_in_term(term: int) -> tuple[int, int]:
    """Gives out months as ints from term as int.

    Args:
        term: term

    Returns:
        tuple: months
    """
    month_term_dict = {
        1: (1, 2),
        2: (3, 4),
        3: (5, 6),
        4: (7, 8),
        5: (9, 10),
        6: (11, 12),
    }

    return month_term_dict[term]


def iterate_years_months(
    start_year: int, end_year: int, start_month: int, end_month: int
) -> Any:
    """Function to iterate over years and month.

    Allows you to select start year, start month, end year and end month

    Args:
        start_year: Int for start year.
        start_month: Int for start month.
        end_year: Int for end year.
        end_month: Int for end month.

    Yields:
        Any: A tuple containing the year and month for each combination.

    Raises:
        ValueError: If start year is bigger than end year.
        ValueError: If month is invalid number.
        ValueError: If end month is bigger than start and only iterating on one year.
    """
    if start_year > end_year:
        raise ValueError("Start year must be less than or equal to end year")
    if start_month < 1 or start_month > 12 or end_month < 1 or end_month > 12:
        raise ValueError("Month must be between 1 and 12")
    if start_year == end_year and start_month > end_month:
        raise ValueError(
            "If iterating in same year start month must be less than end month."
        )

    for year in range(start_year, end_year + 1):
        for month in range(1, 13):
            if (year == start_year and month < start_month) or (
                year == end_year and month > end_month
            ):
                continue
            yield year, month


def validate_month(month: int | str) -> str:
    """Ensure month to have leading zero if before october.

    Args:
        month: the number of the month

    Returns:
        str: the number of the month with leading zero if relevant
    """
    if int(month) < 10:
        month = "0" + str(int(month))
    return str(month)


def validate_day(day: int | str) -> str:
    """Ensure day to have leading zero if it less than 10.

    Args:
        day: the number of the month

    Returns:
        str: the number of the day with leading zero if relevant
    """
    if int(day) < 10:
        day = "0" + str(int(day))
    return str(day)
